//@ts-check
import { Plugin } from "prosemirror-state"
import { keymap } from "prosemirror-keymap"
import {
  buildInputRules,
  buildKeymap,
  buildMenuItems,
} from "prosemirror-example-setup";
import { baseKeymap } from "prosemirror-commands";
import { dropCursor } from "prosemirror-dropcursor"
import { gapCursor } from "prosemirror-gapcursor"
import {
  menuBar,
  MenuItem,
  Dropdown,
  undoItem as prosemirrorUndoItem,
  redoItem as prosemirrorRedoItem,
  liftItem,
} from "prosemirror-menu";
import { yUndoPluginKey } from "y-prosemirror";
/**@typedef {import('prosemirror-model').Schema} Schema */

/**
 * @typedef {{
 *   schema: Schema
 *   undo: import('y-prosemirror').undo,
 *   redo: import('y-prosemirror').redo,
 * }} Options
 */

/**
 * @param {Options} options
 */
export default function getProsemirrorSetup(options) {
  // A modified `prosemirror-example-setup/src/index.ts`.

  const menuItemsBase = buildMenuItems(options.schema);
  const menuItemsFinal = [
    [
      new Dropdown(
        [
          menuItemsBase.wrapBulletList,
          menuItemsBase.wrapOrderedList,
          liftItem,
          menuItemsBase.wrapBlockQuote,
          menuItemsBase.insertHorizontalRule,
          new MenuItem({
            run: () => document.documentElement.classList.toggle('dark'),
            class: 'theme-button',
            title: 'Toggle color theme',
            icon: { text: '' },
          }),
        ], {
          // TODO i18n, and for all other menu items.
          label: '≡',
          title: 'More…',
          class: 'hamburger-menu-item',
        },
      ),
      new Dropdown(
        [
          menuItemsBase.makeParagraph,
          // TODO improvement: change labels from "Level n" to "Heading n".
          menuItemsBase.makeHead1,
          menuItemsBase.makeHead2,
          menuItemsBase.makeHead3,
          menuItemsBase.makeHead4,
          menuItemsBase.makeHead5,
          menuItemsBase.makeHead6,
        ], {
          // TODO i18n, and for all other menu items.
          label: 'Type…',
        },
      ),
      // menuItems.makeParagraph,
      menuItemsBase.toggleStrong,
      menuItemsBase.toggleEm,
      menuItemsBase.toggleCode,
    ],
    [
      new MenuItem({
        run: options.undo,
        // TODO `enable` doesn't right for some reason. I think it's getting
        // called too early, before, say, `redo` finishes.
        // enable: state => getYjsUndoManager(state)?.canUndo() || false,

        icon:  prosemirrorUndoItem.spec.icon,
        label: prosemirrorUndoItem.spec.label,
        title: prosemirrorUndoItem.spec.title,
        class: prosemirrorUndoItem.spec.class,
        css:   prosemirrorUndoItem.spec.css,
      }),
      new MenuItem({
        run: options.redo,
        // enable: state => getYjsUndoManager(state)?.canRedo() || false,

        icon:  prosemirrorRedoItem.spec.icon,
        label: prosemirrorRedoItem.spec.label,
        title: prosemirrorRedoItem.spec.title,
        class: prosemirrorRedoItem.spec.class,
        css:   prosemirrorRedoItem.spec.css,
      }),
    ],
    // TODO horizontal divider, other useful stuff.
  ];

  const plugins = [
    buildInputRules(options.schema),
    keymap(buildKeymap(options.schema, undefined)),
    keymap(baseKeymap),
    dropCursor(),
    gapCursor(),
    menuBar({
      // TODO `floating` doesn't work the way we want it to on iOS. See
      // https://codeberg.org/webxdc/editor/issues/11
      // Also maybe just replace it with `position: sticky`, like here:
      // https://codeberg.org/webxdc/editor/pulls/32
      floating: true,
      // content: menuItems.fullMenu,
      content: menuItemsFinal,
    }),
  ]
  return plugins.concat(new Plugin({
    props: {
      attributes: {class: "ProseMirror-example-setup-style"}
    }
  }))
}

// Looked up in `y-prosemirror/src/plugins/undo-plugin.js
function getYjsUndoManager(state) {
  const undoManager = /**@type {import('yjs').UndoManager | undefined}*/(
    yUndoPluginKey.getState(state).undoManager
  );
  return undoManager;
}
